// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/driver/ioctl.c $
// $Rev: 52358 $
// $Date: 2023-01-12 10:04:31 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTO_CAL_START			D28			// BCR
#define	AUTO_CAL_PASS			D29			// BCR
#define	INIT_START				D31			// BCR

#define	IRQ_AUTO_CAL_DONE		0x00000001L	// PSR



//*****************************************************************************
static void _psr_mod(dev_data_t* dev, u32 value, u32 mask)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_psr_32, value & 0xFFFF, mask | 0xFFFF0000);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			arg[0]	= AOF16C500KR_IOCTL_QUERY_ERROR;
			ret		= -EINVAL;
			break;

		case AOF16C500KR_QUERY_AO_CONFIG:		arg[0]	= dev->cache.output_config;			break;
		case AOF16C500KR_QUERY_AUTO_CAL_MS:		arg[0]	= dev->cache.auto_cal_ms;			break;
		case AOF16C500KR_QUERY_CHANNEL_MASK:	arg[0]	= dev->cache.channel_mask;			break;
		case AOF16C500KR_QUERY_CHANNEL_MAX:		arg[0]	= dev->cache.channels_max;			break;
		case AOF16C500KR_QUERY_CHANNEL_QTY:		arg[0]	= dev->cache.channel_qty;			break;
		case AOF16C500KR_QUERY_COUNT:			arg[0]	= AOF16C500KR_QUERY_LAST;			break;
		case AOF16C500KR_QUERY_CUSTOM_FEAT:		arg[0]	= dev->cache.custom_features;		break;
		case AOF16C500KR_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;					break;
		case AOF16C500KR_QUERY_FGEN_MAX:		arg[0]	= dev->cache.fgen_max;				break;
		case AOF16C500KR_QUERY_FGEN_MIN:		arg[0]	= dev->cache.fgen_min;				break;
		case AOF16C500KR_QUERY_FIFO_SIZE:		arg[0]	= dev->cache.fifo_size;				break;
		case AOF16C500KR_QUERY_FILTER:			arg[0]	= dev->cache.filter;				break;
		case AOF16C500KR_QUERY_FREF_DEFAULT:	arg[0]	= dev->cache.fref_default;			break;
		case AOF16C500KR_QUERY_FSAMP_MAX:		arg[0]	= dev->cache.fsamp_max;				break;
		case AOF16C500KR_QUERY_FSAMP_MIN:		arg[0]	= dev->cache.fsamp_min;				break;
		case AOF16C500KR_QUERY_INIT_MS:			arg[0]	= dev->cache.initialize_ms;			break;
		case AOF16C500KR_QUERY_NDIV_MAX:		arg[0]	= dev->cache.ndiv_max;				break;
		case AOF16C500KR_QUERY_NDIV_MIN:		arg[0]	= dev->cache.ndiv_min;				break;
		case AOF16C500KR_QUERY_RANGE:			arg[0]	= dev->cache.range;					break;
	}

	return(ret);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	u32		bcr;
	int		i;
	long	ms	= dev->cache.initialize_ms + 5000;
	int		ret	= 0;
	int		tmp;

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, INIT_START);

	// There is no Initialize Done interrupt so we wait manually.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(dev_data_t* dev, void* arg)
{
	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, AUTO_CAL_START);
	return(0);
}



//*****************************************************************************
static int _auto_calibrate(dev_data_t* dev, void* arg)
{
	u32			bcr;
	u32			mask;
	long		ms		= dev->cache.auto_cal_ms + 5000;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Auto-Cal Done interrupt.
	mask	= IRQ_AUTO_CAL_DONE;
	value	= IRQ_AUTO_CAL_DONE;
	_psr_mod(dev, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AOF16C500KR_WAIT_GSC_AUTO_CAL_DN;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & AUTO_CAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if ((bcr & AUTO_CAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_sts(dev_data_t* dev, s32* arg)
{
	u32	bcr;

	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & D28)
		arg[0]	= AOF16C500KR_AUTO_CAL_STS_ACTIVE;
	else if (bcr & D29)
		arg[0]	= AOF16C500KR_AUTO_CAL_STS_PASS;
	else
		arg[0]	= AOF16C500KR_AUTO_CAL_STS_FAIL;

	return(0);
}



//*****************************************************************************
static int _buf_clear(dev_data_t* dev, s32* arg)
{
	int	ret;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D11, D11);
	ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_boor_32, D11, 0);

	// Clear the overrun flags.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, 0, D16 | D17);
	return(ret);
}



//*****************************************************************************
static int _buf_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BUF_MODE_OPEN,
		AOF16C500KR_BUF_MODE_CIRC,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _buf_over_data(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BUF_OVER_DATA_NO,
		AOF16C500KR_BUF_OVER_DATA_YES,
		-1	// terminate list
	};

	int	ret;

	if ((arg[0] == AOF16C500KR_BUF_OVER_DATA_CLR) ||
		(arg[0] == AOF16C500KR_BUF_OVER_DATA_CHK))
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 16, 16);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _buf_over_frame(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BUF_OVER_FRAME_NO,
		AOF16C500KR_BUF_OVER_FRAME_YES,
		-1	// terminate list
	};

	int	ret;

	if ((arg[0] == AOF16C500KR_BUF_OVER_FRAME_CLR) ||
		(arg[0] == AOF16C500KR_BUF_OVER_FRAME_CHK))
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 17, 17);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _buf_status(dev_data_t* dev, s32* arg)
{
	u32	boor;

	boor	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_boor_32);

	if (boor & D12)
		arg[0]	= AOF16C500KR_BUF_STATUS_EMPTY;
	else if (boor & D15)
		arg[0]	= AOF16C500KR_BUF_STATUS_FULL;
	else if (boor & D13)
		arg[0]	= AOF16C500KR_BUF_STATUS_AF;
	else
		arg[0]	= AOF16C500KR_BUF_STATUS_AE;

	return(0);
}



//*****************************************************************************
static int _buf_size(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, _256K, dev->vaddr.gsc_obsr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _buf_thresh(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_reg(dev, arg, 0, _256K, dev->vaddr.gsc_obtr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BUF_THR_STS_CLR,
		AOF16C500KR_BUF_THR_STS_SET,
		-1	// terminate list
	};

	int	ret;

	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_obtr_32, 20, 20);
	return(ret);
}



//*****************************************************************************
static int _burst_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BURST_ENABLE_NO,
		AOF16C500KR_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _burst_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BURST_READY_NO,
		AOF16C500KR_BURST_READY_YES,
		-1	// terminate list
	};

	int		ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _burst_trig_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_BURST_TRIG_SRC_RAG,
		AOF16C500KR_BURST_TRIG_SRC_RBG,
		AOF16C500KR_BURST_TRIG_SRC_EXT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ocr_32, 21, 20);
	return(ret);
}



//*****************************************************************************
static int _channel_sel(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			0,
			dev->cache.channel_mask,
			dev->vaddr.gsc_ocr_32,
			15, 0);
	return(ret);
}



//*****************************************************************************
static int _clock_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_CLOCK_ENABLE_NO,
		AOF16C500KR_CLOCK_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _clock_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_CLOCK_READY_NO,
		AOF16C500KR_CLOCK_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _clock_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_CLOCK_SRC_RAG,
		AOF16C500KR_CLOCK_SRC_RBG,
		AOF16C500KR_CLOCK_SRC_EXT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ocr_32, 19, 18);
	return(ret);
}



//*****************************************************************************
static int _clock_sw(dev_data_t* dev, s32* arg)
{
	int	ret;

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D7, D7);
	ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_boor_32, D7, 0);
	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_DATA_FORMAT_2S_COMP,
		AOF16C500KR_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 25, 25);
	return(ret);
}



//*****************************************************************************
static int _dio_dir_out(dev_data_t* dev, s32* arg)
{
	u32	diopr;
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			diopr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_diopr_32);
			arg[0]	= diopr & (D15 | D7);
			break;

		case AOF16C500KR_DIO_DIR_OUT_NONE:
		case AOF16C500KR_DIO_DIR_OUT_D3_D0:
		case AOF16C500KR_DIO_DIR_OUT_D7_D4:
		case AOF16C500KR_DIO_DIR_OUT_D7_D0:

			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_diopr_32, arg[0], D15 | D7);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _dio_read(dev_data_t* dev, s32* arg)
{
	u32	diopr;

	diopr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_diopr_32);
	arg[0]	= ((diopr & 0xF00) >> 4) | (diopr & 0xF);
	return(0);
}



//*****************************************************************************
static int _dio_write(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	v;

	if (arg[0] == -1)
	{
		ret	= _dio_read(dev, arg);
	}
	else if ((arg[0] < 0) || (arg[0] > 0xFF))
	{
		ret	= -EINVAL;
	}
	else
	{
		v	= ((arg[0] << 4) | arg[0])  & 0xF0F;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_diopr_32, v, 0xF0F);
	}

	return(ret);
}



//*****************************************************************************
static int _filter(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_FILTER_DISABLE,
		AOF16C500KR_FILTER_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 24, 24);
	return(ret);
}



//*****************************************************************************
static int _gen_a_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_GEN_ENABLE_NO,
		AOF16C500KR_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 21, 21);
	return(ret);
}



//*****************************************************************************
static int _gen_a_ndiv(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_ragr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _gen_b_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_GEN_ENABLE_NO,
		AOF16C500KR_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 22, 22);
	return(ret);
}



//*****************************************************************************
static int _gen_b_ndiv(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_rbgr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _ground_sense(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_GROUND_SENSE_DISABLE,
		AOF16C500KR_GROUND_SENSE_REMOTE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _irq_sel(dev_data_t* dev, s32* arg)
{
	u32	psr;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		psr		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_psr_32);
		arg[0]	= psr & AOF16C500KR_IRQ_ALL;
	}
	else if ((arg[0] & AOF16C500KR_IRQ_ALL) != arg[0])
	{
		ret	= -EINVAL;
	}
	else
	{
		_psr_mod(dev, arg[0], AOF16C500KR_IRQ_ALL);
	}

	return(ret);
}



//*****************************************************************************
static int _led(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_LED_OFF,
		AOF16C500KR_LED_ON,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _load_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_LOAD_READY_NO,
		AOF16C500KR_LOAD_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_boor_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _load_request(dev_data_t* dev, s32* arg)
{
	long		ms;
	int			ret;

	// Make the request.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D9, D9);

	// Manually wait for completion.
	ms	= dev->io.tx.timeout_s * 1000;
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_boor_32, D10, D10);
	return(ret);
}



//*****************************************************************************
static int _output_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_OUTPUT_MODE_SEQ,
		AOF16C500KR_OUTPUT_MODE_SIM,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _range_sub(dev_data_t* dev, s32* arg, const u32* list)
{
	u32	ocr;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		ocr		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ocr_32);
		arg[0]	= (ocr & D16) ? list[1] : list[0];
	}
	else if (arg[0] == list[0])
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ocr_32, 0, D16);
	}
	else if (arg[0] == list[1])
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ocr_32, D16, D16);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _range(dev_data_t* dev, s32* arg)
{
	//									low voltage			 high voltage
	static const u32	_6_3[2]		= { AOF16C500KR_RANGE_3, AOF16C500KR_RANGE_6 };
	static const u32	_10_5[2]	= { AOF16C500KR_RANGE_5, AOF16C500KR_RANGE_10 };

	int	ret;

	switch (dev->cache.range)
	{
		default:

				ret	= -EINVAL;
				break;

		case AOF16C500KR_RANGE_6_3:

			ret	= _range_sub(dev, arg, _6_3);
			break;

		case AOF16C500KR_RANGE_10_5:

			ret	= _range_sub(dev, arg, _10_5);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _trigger_sw(dev_data_t* dev, s32* arg)
{
	int	ret;

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_boor_32, D20, D20);
	ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_boor_32, D20, 0);
	return(ret);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_ovr_dat(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			arg[0]	= dev->io.tx.overflow_data;
			break;

		case AOF16C500KR_TX_IO_OVR_DAT_IGNORE:
		case AOF16C500KR_TX_IO_OVR_DAT_CHECK:

			dev->io.tx.overflow_data	= arg[0];
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_io_ovr_frm(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			arg[0]	= dev->io.tx.overflow_frame;
			break;

		case AOF16C500KR_TX_IO_OVR_FRM_IGNORE:
		case AOF16C500KR_TX_IO_OVR_FRM_CHECK:

			dev->io.tx.overflow_frame	= arg[0];
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AOF16C500KR_IO_TIMEOUT_MIN,
			AOF16C500KR_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _xcvr_output(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_XCVR_OUTPUT_DISABLE,
		AOF16C500KR_XCVR_OUTPUT_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ocr_32, 17, 17);
	return(ret);
}



//*****************************************************************************
static int _xcvr_pol(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_XCVR_POL_DEF,
		AOF16C500KR_XCVR_POL_INV,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ocr_32, 23, 23);
	return(ret);
}



//*****************************************************************************
static int _xcvr_type(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AOF16C500KR_XCVR_TYPE_TTL,
		AOF16C500KR_XCVR_TYPE_LVDS,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 5);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AOF16C500KR_IOCTL_REG_READ,		(void*) gsc_reg_read_ioctl		},
	{ AOF16C500KR_IOCTL_REG_WRITE,		(void*) gsc_reg_write_ioctl		},
	{ AOF16C500KR_IOCTL_REG_MOD,		(void*) gsc_reg_mod_ioctl		},
	{ AOF16C500KR_IOCTL_QUERY,			(void*) _query					},
	{ AOF16C500KR_IOCTL_INITIALIZE,		(void*) initialize_ioctl		},
	{ AOF16C500KR_IOCTL_AUTO_CALIBRATE,	(void*) _auto_calibrate			},
	{ AOF16C500KR_IOCTL_AUTO_CAL_STS,	(void*) _auto_cal_sts			},
	{ AOF16C500KR_IOCTL_BUF_CLEAR,		(void*) _buf_clear				},
	{ AOF16C500KR_IOCTL_BUF_MODE,		(void*) _buf_mode				},
	{ AOF16C500KR_IOCTL_BUF_OVER_DATA,	(void*) _buf_over_data			},
	{ AOF16C500KR_IOCTL_BUF_OVER_FRAME,	(void*) _buf_over_frame			},
	{ AOF16C500KR_IOCTL_BUF_STATUS,		(void*) _buf_status				},
	{ AOF16C500KR_IOCTL_BUF_SIZE,		(void*) _buf_size				},
	{ AOF16C500KR_IOCTL_BUF_THRESH,		(void*) _buf_thresh				},
	{ AOF16C500KR_IOCTL_BUF_THR_STS,	(void*) _buf_thr_sts			},
	{ AOF16C500KR_IOCTL_BURST_ENABLE,	(void*) _burst_enable			},
	{ AOF16C500KR_IOCTL_BURST_READY,	(void*) _burst_ready			},
	{ AOF16C500KR_IOCTL_BURST_TRIG_SRC,	(void*) _burst_trig_src			},
	{ AOF16C500KR_IOCTL_CHANNEL_SEL,	(void*) _channel_sel			},
	{ AOF16C500KR_IOCTL_CLOCK_ENABLE,	(void*) _clock_enable			},
	{ AOF16C500KR_IOCTL_CLOCK_READY,	(void*) _clock_ready			},
	{ AOF16C500KR_IOCTL_CLOCK_SRC,		(void*) _clock_src				},
	{ AOF16C500KR_IOCTL_CLOCK_SW,		(void*) _clock_sw				},
	{ AOF16C500KR_IOCTL_DATA_FORMAT,	(void*) _data_format			},
	{ AOF16C500KR_IOCTL_DIO_DIR_OUT,	(void*) _dio_dir_out			},
	{ AOF16C500KR_IOCTL_DIO_READ,		(void*) _dio_read				},
	{ AOF16C500KR_IOCTL_DIO_WRITE,		(void*) _dio_write				},
	{ AOF16C500KR_IOCTL_FILTER,			(void*) _filter					},
	{ AOF16C500KR_IOCTL_GEN_A_ENABLE,	(void*) _gen_a_enable			},
	{ AOF16C500KR_IOCTL_GEN_A_NDIV,		(void*) _gen_a_ndiv				},
	{ AOF16C500KR_IOCTL_GEN_B_ENABLE,	(void*) _gen_b_enable			},
	{ AOF16C500KR_IOCTL_GEN_B_NDIV,		(void*) _gen_b_ndiv				},
	{ AOF16C500KR_IOCTL_GROUND_SENSE,	(void*) _ground_sense			},
	{ AOF16C500KR_IOCTL_IRQ_SEL,		(void*) _irq_sel				},
	{ AOF16C500KR_IOCTL_LED,			(void*) _led					},
	{ AOF16C500KR_IOCTL_LOAD_READY,		(void*) _load_ready				},
	{ AOF16C500KR_IOCTL_LOAD_REQUEST,	(void*) _load_request			},
	{ AOF16C500KR_IOCTL_OUTPUT_MODE,	(void*) _output_mode			},
	{ AOF16C500KR_IOCTL_RANGE,			(void*) _range					},
	{ AOF16C500KR_IOCTL_TRIGGER_SW,		(void*) _trigger_sw				},
	{ AOF16C500KR_IOCTL_TX_IO_ABORT,	(void*) _tx_io_abort			},
	{ AOF16C500KR_IOCTL_TX_IO_MODE,		(void*) _tx_io_mode				},
	{ AOF16C500KR_IOCTL_TX_IO_OVR_DAT,	(void*) _tx_io_ovr_dat			},
	{ AOF16C500KR_IOCTL_TX_IO_OVR_FRM,	(void*) _tx_io_ovr_frm			},
	{ AOF16C500KR_IOCTL_TX_IO_TIMEOUT,	(void*) _tx_io_timeout			},
	{ AOF16C500KR_IOCTL_WAIT_CANCEL,	(void*) gsc_wait_cancel_ioctl	},
	{ AOF16C500KR_IOCTL_WAIT_EVENT,		(void*) gsc_wait_event_ioctl	},
	{ AOF16C500KR_IOCTL_WAIT_STATUS,	(void*) gsc_wait_status_ioctl	},
	{ AOF16C500KR_IOCTL_XCVR_OUTPUT,	(void*) _xcvr_output			},
	{ AOF16C500KR_IOCTL_XCVR_POL,		(void*) _xcvr_pol				},
	{ AOF16C500KR_IOCTL_XCVR_TYPE,		(void*) _xcvr_type				},
	{ -1, NULL }
};


