// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/20AOF16C500KR/samples/aout/perform.c $
// $Rev: 52364 $
// $Date: 2023-01-12 10:13:23 -0600 (Thu, 12 Jan 2023) $

// 20AOF16C500KR: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



//*****************************************************************************
static void _init_data(const args_t* args, int chans, int rate)
{
	#define	PIE		(3.1415962)

	u32*		dest;
	int			i;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	long		samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) rate) * chans;
	samp_tot	= ((float) rate) * chans * args->seconds;
	samp_dat	= sizeof(_data) / 4;
	period_len	= (rate <= 100)		? rate			//      10-      100 ->    10-   100
				: (rate <= 1000)	? (rate / 2)	//     101-    1,000 ->    50-   500
				: (rate <= 10000)	? (rate / 5)	//   1,001-   10,000 ->   200- 2,000
				: (rate <= 100000)	? (rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (rate / 100);						// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= args->seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= args->seconds * (samp_sec / period_tot);
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (samp_dat / chans) * chans;
		_loops	= (samp_tot + _qty - 1) / _qty;
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= 0xFFFFFL - (0xFFFFFL * samp / (period_len - 1));

		// 2nd channel: ramp wave
		*dest++	= 0xFFFFFL * samp / (period_len - 1);

		// 3rd channel: square wave
		*dest++	= (samp < (period_len / 2)) ? 0xFFFFFL : 0;

		// 4th channel: sine wave
		*dest++	= 0x7FFFFL * (1 + sin((2 * PIE / period_len) * samp));
	}

	printf("DONE  (pattern length: %ld samples, %ld %s)\n", period_len, period, units);
}



//*****************************************************************************
static int _io_write(const args_t* args, int errs)
{
	os_time_ns_t	begin;
	long			loop;
	long long		ms;
	os_time_ns_t	now;
	char*			ptr;
	int				ret;
	int				send;
	int				sent	= 0;
	s32				status;
	long long		total	= 0;

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED  (due to errors)\n");
			break;
		}

		errs	= 0;
		os_time_get_ns(&begin);

		// Write the data to the device.

		for (loop = 0; loop < _loops; loop++)
		{
			send	= _qty * 4;
			ptr		= (char*) &_data;

			for (;;)
			{
				sent	= aof16c500kr_write(args->fd, ptr, send);

				if (sent <= 0)
					break;

				total	+= sent;

				if (sent == send)
					break;

				ptr		+= sent;
				send	-= sent;
			}

			if (sent <= 0)
				break;
		}

		// Wait for the output buffer to empty.

		for (;;)
		{
			ret		= aof16c500kr_ioctl(args->fd, AOF16C500KR_IOCTL_BUF_STATUS, &status);
			errs	+= ret ? 1 : 0;

			if ((errs) || (status == AOF16C500KR_BUF_STATUS_EMPTY))
				break;
		}

		// Compute the results.
		os_time_get_ns(&now);

		if (sent == -1)
		{
			errs	= 1;
			printf("FAIL <---  (write error)\n");
		}
		else
		{
			if (sent == 0)
				printf("FAIL <---  (I/O timeout, ");
			else
				printf("PASS  (");

			ms		= ((long long) now.tv_sec * 1000 + now.tv_nsec / 1000000)
					- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);
			gsc_label_long_comma((long) total);
			printf(" samples, %lld.%03lld Seconds, ", ms / 1000, ms % 1000);
			gsc_label_long_comma((long) (total / ms * 1000 / 4));
			printf(" S/S)\n");
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	s32	rate	= args->rate;

	errs	+= aof16c500kr_config_ao			(args->fd, -1, 1, rate);
	errs	+= aof16c500kr_tx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= aof16c500kr_channel_sel			(args->fd, -1, 1, 0xF, NULL);
	errs	+= aof16c500kr_fsamp_ao_report_all	(args->fd, -1, 1, NULL);

	gsc_label("Analog Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aof16c500kr_fsamp_ao_report(args->fd, -1, 0, 1, &rate);
	_init_data(args, 4, rate);
	errs	+= aof16c500kr_buf_over_frame(args->fd, -1, 1, AOF16C500KR_BUF_OVER_FRAME_CLR, NULL);
	errs	+= aof16c500kr_buf_over_data(args->fd, -1, 1, AOF16C500KR_BUF_OVER_DATA_CLR, NULL);
	errs	+= _io_write(args, errs);

	gsc_label_level_dec();
	return(errs);
}



